@@ -1,4 +1,5 @@
-from typing import Union, Optional, List
+import builtins
+from typing import Union, Optional, List, Any
 
 import astroid
 from pylint.checkers import BaseChecker, utils
@@ -95,6 +96,34 @@ def _get_protobuf_descriptor(node):
     return cls_def._protobuf_descriptor  # type: SimpleDescriptor
 
 
+def _scalar_typecheck(val, val_type):
+    # type: (Union[type, Any], type) -> bool
+    # NOTE: transform.to_pytype returns {bool, int, float, str, <other>}
+    if type(val) is type:
+        check = issubclass
+    else:
+        check = isinstance
+    if val_type is float:
+        return check(val, (float, int, bool))
+    elif val_type is int:
+        return check(val, (int, bool))
+    elif val_type is bool:
+        return check(val, (float, int, bool))
+    elif val_type is str:
+        return check(val, str)
+    else:
+        return True  # Are there any other scalar protobuf types?
+
+
+def _resolve_builtin(inst):
+    # type: (astroid.Instance) -> Optional[type]
+    typename = inst.pytype()
+    if not typename.startswith('builtins.'):
+        return None
+    typename = typename[len('builtins.'):]
+    return getattr(builtins, typename, None)  # eh...
+
+
 class ProtobufDescriptorChecker(BaseChecker):
     __implements__ = IAstroidChecker
     msgs = MESSAGES
@@ -177,8 +206,10 @@ class ProtobufDescriptorChecker(BaseChecker):
                 # messages
                 for val in _get_inferred_values(val_node):
                     if isinstance(val, astroid.Const):
-                        self.add_message('protobuf-type-error', node=node,
-                                         args=(desc.name, arg_name, arg_type.__name__, val.value))
+                        if val.value is not None:
+                            # Special-case None as default of keyword args
+                            self.add_message('protobuf-type-error', node=node,
+                                             args=(desc.name, arg_name, arg_type.__name__, val.value))
                         break
                     val_desc = _get_protobuf_descriptor(val)
                     if val_desc is None:
@@ -189,7 +220,20 @@ class ProtobufDescriptorChecker(BaseChecker):
                                          args=(desc.name, arg_name, arg_type.__name__, val))
             else:
                 for val_const in _get_inferred_values(val_node):
-                    if not hasattr(val_const, 'value'):
+                    if hasattr(val_const, 'value'):
+                        val = val_const.value
+                    elif hasattr(val_const, '_proxied'):
+                        # check for <Instance of builtins.int> etc.
+                        val = _resolve_builtin(val_const)
+                        if val is None:  # check for composite assigned to scalar
+                            try:
+                                val = '{}()'.format(val_const.name)
+                            except AttributeError:
+                                continue
+                            self.add_message('protobuf-type-error', node=node,
+                                             args=(desc.name, arg_name, arg_type.__name__, val))
+                            break
+                    else:
                         try:
                             val = '{}()'.format(val_const.name)
                         except AttributeError:
@@ -197,10 +241,11 @@ class ProtobufDescriptorChecker(BaseChecker):
                         self.add_message('protobuf-type-error', node=node,
                                          args=(desc.name, arg_name, arg_type.__name__, val))
                         break
-                    val = val_const.value
-                    if not isinstance(val, arg_type):
-                        self.add_message('protobuf-type-error', node=node,
-                                         args=(desc.name, arg_name, arg_type.__name__, val))
+                    if not _scalar_typecheck(val, arg_type):
+                        if val is not None:
+                            # Special-case None as default of keyword args
+                            self.add_message('protobuf-type-error', node=node,
+                                             args=(desc.name, arg_name, arg_type.__name__, val))
                         break
 
     @utils.check_messages('protobuf-type-error')
@@ -254,7 +299,7 @@ class ProtobufDescriptorChecker(BaseChecker):
             return  # warn?
 
         def check_arg(val):
-            if not isinstance(val, arg_type):
+            if not _scalar_typecheck(val, arg_type):
                 self.add_message('protobuf-type-error', node=node,
                                  args=(desc.name, arg_name, arg_type.__name__, val))
         for val in vals:
@@ -319,7 +364,7 @@ class ProtobufDescriptorChecker(BaseChecker):
             if not hasattr(val_const, 'value'):
                 continue
             val = val_const.value
-            if not isinstance(val, type_):
+            if not _scalar_typecheck(val, type_):
                 self.add_message('protobuf-type-error', node=node, args=(desc.name, attr, type_.__name__, val))
                 break
 
